unit MainUnit;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls,DateUtils, Vcl.ExtCtrls;

type
  TfrmMain = class(TForm)
    memLog: TMemo;
    btnSort: TButton;
    OpenDialog1: TOpenDialog;
    edFolder: TEdit;
    Label1: TLabel;
    btnChooseFolder: TButton;
    btnClose: TButton;
    Label2: TLabel;
    Label3: TLabel;
    chkYear: TCheckBox;
    chkMonth: TCheckBox;
    chkDay: TCheckBox;
    Label4: TLabel;
    chkMoveFiles: TCheckBox;
    procedure Button1Click(Sender: TObject);
    procedure btnSortClick(Sender: TObject);
    procedure btnChooseFolderClick(Sender: TObject);
    procedure btnCloseClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    { Private declarations }
    procedure ReportTime(const Name: string; const FileTime: TFileTime);
    function  CreateFolder (const Folder : string) : Boolean;
  public
    { Public declarations }
  end;


function ExeVersion : string;

var
  frmMain: TfrmMain;
  ExeVerStr        : String;

implementation

{$R *.dfm}

procedure TfrmMain.btnChooseFolderClick(Sender: TObject);
begin
  // aleg folderul de sortat
  if OpenDialog1.Execute then
  begin
    edFolder.Text:=ExtractFileDir(OpenDialog1.FileName);
  end;
end;

procedure TfrmMain.btnCloseClick(Sender: TObject);
begin
  Close;
end;

procedure TfrmMain.Button1Click(Sender: TObject);
var
  fad: TWin32FileAttributeData;
  FileName : string;
begin
{
  FileName := 'C:\2.jpg';
  if not GetFileAttributesEx(PChar(FileName), GetFileExInfoStandard, @fad) then
    RaiseLastOSError;
  Memo1.Clear;
  Memo1.Lines.Add(FileName);
  ReportTime('Created', fad.ftCreationTime);
  ReportTime('Modified', fad.ftLastWriteTime);
  ReportTime('Accessed', fad.ftLastAccessTime);
  }
end;

function TfrmMain.CreateFolder(const Folder : string): Boolean;
begin
  if not DirectoryExists(Folder) then
    Result := CreateDir(Folder)
  else
    Result := True;
end;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  Caption := Caption + ' ' + ExeVersion;
end;

procedure TfrmMain.btnSortClick(Sender: TObject);
var
  Path, NewPath,
  FileName  : String;
  SR        : TSearchRec;
  DirList   : TStrings;
  I,An,Luna,Zi
            : Integer;
  SystemTime, LocalTime: TSystemTime;
  fad: TWin32FileAttributeData;
begin
  // iau path-ul pentru fisierul ales
  Path:= edFolder.Text;
  DirList:=TStringList.Create;
  try
    // iau lista de fisiere
    if FindFirst(Path + '\*', faAnyFile, SR) = 0 then
    begin
      repeat
        DirList.Add(SR.Name); //Fill the list
      until FindNext(SR) <> 0;
      FindClose(SR);
    end;
    // parcurg lista de fisiere
    // 0 si 1 sunt . si ..
    for I := 2 to DirList.Count-1 do
    begin
      FileName := DirList[I];
      if not GetFileAttributesEx(PChar(Path+'\'+FileName), GetFileExInfoStandard, @fad) then
        RaiseLastOSError;
      if not FileTimeToSystemTime(fad.ftLastWriteTime, SystemTime) then
        RaiseLastOSError;
      if not SystemTimeToTzSpecificLocalTime(nil, SystemTime, LocalTime) then
        RaiseLastOSError;

      // iau anul, luna si ziua
      An    := YearOf(SystemTimeToDateTime(LocalTime));
      Luna  := MonthOf(SystemTimeToDateTime(LocalTime));
      Zi    := DayOf(SystemTimeToDateTime(LocalTime));

       // creez calea cea noua de directoare
      NewPath := Path;
      if chkYear.Checked then
      begin
        if CreateFolder(NewPath + '\'+IntToStr(An)) then
          NewPath := NewPath + '\'+ IntToStr(An);
      end;

      if chkMonth.Checked then
      begin
        if CreateFolder(NewPath + '\'+IntToStr(Luna)) then
          NewPath := NewPath + '\'+ IntToStr(Luna);
      end;

      if chkDay.Checked then
      begin
        if CreateFolder(NewPath + '\'+IntToStr(Zi)) then
          NewPath := NewPath + '\'+ IntToStr(Zi);
      end;

      // fac mutarea fisierului din 2 pasi: copy si delete
      CopyFile(PChar(Path+'\'+FileName),PChar(NewPath+'\'+DirList[I]),True);
      // daca e bifata partea de mutare fisiere - fac stergerea
      if chkMoveFiles.Checked then
        DeleteFile(Path+'\'+FileName);

      // scriu in log
      memLog.Lines.Add(FileName);
      memLog.Lines.Add(Name + ': ' + DateTimeToStr(SystemTimeToDateTime(LocalTime)));
    end;
   //do your stuff

  finally
   DirList.Free;
  end;

end;

procedure TfrmMain.ReportTime(const Name: string; const FileTime: TFileTime);
var
    SystemTime, LocalTime: TSystemTime;
begin
{
    if not FileTimeToSystemTime(FileTime, SystemTime) then
      RaiseLastOSError;
    if not SystemTimeToTzSpecificLocalTime(nil, SystemTime, LocalTime) then
      RaiseLastOSError;
    Memo1.Lines.Add(Name + ': ' + DateTimeToStr(SystemTimeToDateTime(LocalTime)));
    }
end;

function ExeVersion: String;
var ExePChar  : PChar;
    aHandle   : DWord;
    aSize     : Integer;
    lpData    : Pointer;
    lpBuffer  : Pointer;
    lpSize    : DWord;
    LangStr   : String;
    LangId    : DWord;
begin
  if ExeVerStr = '' then begin
     ExePChar := PChar(ParamStr(0));
     aSize := GetFileVersionInfoSize(ExePChar, aHandle);
     if aSize = 0 then Exit;//RaiseLastWin32Error;
     lpData := AllocMem(aSize);
     try
       if not GetFileVersionInfo(ExePChar, 0, aSize, lpData) then
          Exit;//Abort;
       if not VerQueryValue(lpData, '\VarFileInfo\Translation', lpBuffer, lpSize) then
          Exit;//Abort;
       LangId  := PDWord(lpBuffer)^;
       LangStr := Format('%.4x%.4x',[LangId and $0FFFF,LangId shr 16]);
       if not VerQueryValue(lpData, PChar('\StringFileInfo\'+LangStr+'\FileVersion'), lpBuffer, lpSize) then
          Exit;//Abort;
       ExeVerStr := StrPas(PChar(lpBuffer));
     finally
       FreeMem(lpData, aSize);
     end;
  end;
  Result := ExeVerStr;
end;


end.
